unit Unit1;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, VclTee.TeeGDIPlus, Vcl.StdCtrls,
  Vcl.ExtCtrls, VCLTee.TeEngine, VCLTee.TeeProcs, VCLTee.Chart, VCLTee.Series,
  VCLTee.TeeTools, VCLTee.imapoint, Vcl.Samples.Spin, Vcl.ComCtrls, VCLTee.TeCanvas;
type
  TForm1 = class(TForm)
    Chart1: TChart;
    Panel1: TPanel;
    btnReload: TButton;
    Series1: TBarSeries;
    spnNumValues: TSpinEdit;
    Label1: TLabel;
    cltMarketHit: TColorLineTool;
    btnToWMF: TButton;
    PageControl1: TPageControl;
    tsChart: TTabSheet;
    TabSheet2: TTabSheet;
    Image1: TImage;
    procedure btnReloadClick(Sender: TObject);
    procedure btnToWMFClick(Sender: TObject);
    procedure Chart1MouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure cltMarketHitBeginDragLine(Sender: TColorLineTool);
    procedure cltMarketHitEndDragLine(Sender: TColorLineTool);
    procedure spnIconValueChange(Sender: TObject);
    procedure spnNumValuesChange(Sender: TObject);
    procedure TabSheet2Show(Sender: TObject);
  private
    { Private declarations }
    FsrsPoint: TImagePointSeries;
    procedure LoadSeries;
    procedure UpdateSeriesIcon;
    procedure RenderImage;
  public
    { Public declarations }
    constructor Create(aOwner: TComponent); override;
  end;

var
  Form1: TForm1;

implementation

uses
  Vcl.Imaging.pngimage, System.Math;

{$R *.dfm}

procedure TForm1.btnReloadClick(Sender: TObject);
begin
  LoadSeries;
end;

procedure TForm1.Chart1MouseMove(Sender: TObject; Shift: TShiftState; X, Y:
    Integer);
begin
  FsrsPoint.Marks.Visible := FsrsPoint.Clicked(X,Y)>-1;
end;

constructor TForm1.Create(aOwner: TComponent);
begin
  inherited;
  PageControl1.ActivePageIndex := 0;
  FsrsPoint := nil;
  LoadSeries;
end;

procedure TForm1.cltMarketHitBeginDragLine(Sender: TColorLineTool);
begin
  FsrsPoint.Visible := false;
end;

procedure TForm1.cltMarketHitEndDragLine(Sender: TColorLineTool);
var
  lMax: integer;
begin
  if (Chart1.SeriesCount > 0) then begin
    cltMarketHit.Value := Round(cltMarketHit.Value);

    lMax := Chart1[0].Count - 1;
    if cltMarketHit.Value < 0 then
      cltMarketHit.Value := 0
    else if cltMarketHit.Value > lMax then
      cltMarketHit.Value := lMax;
    UpdateSeriesIcon;
  end;

end;

procedure TForm1.LoadSeries;
var i: Integer;
  lNumValues: Integer;
begin
  Chart1.FreeAllSeries;

  FsrsPoint := nil;

  lNumValues := spnNumValues.Value;
  for i:=0 to 4 do
    with Chart1.AddSeries(TBarSeries) as TBarSeries do begin
      FillSampleValues(lNumValues);
      MultiBar:=mbStacked;
      Marks.Visible:=False;
    end;

  UpdateSeriesIcon;

  Chart1.Axes.Bottom.LabelStyle := talValue;
end;

procedure TForm1.spnIconValueChange(Sender: TObject);
begin
  UpdateSeriesIcon;
end;

procedure TForm1.spnNumValuesChange(Sender: TObject);
begin
  LoadSeries;
end;

procedure AssignChartToGraphic(aChart: TChart; const aRect: TRect; aGraphic: TGraphic; aTransparent: boolean = false);
var
  lHoldShadow: boolean;
  I: Integer;
  lBoolArray: array of boolean;
  lHoldBevel: TBevelCut;
  lHoldColor: TColor;
  lHoldGradientVisible: boolean;
  lMetafile: TMetafile;
  {$ifopt C+}
  lCanvasClassName: string;
  {$endif}
  lBmp: TBitmap;
  lCanvasClass: TCanvas3DClass;
  lHoldHover: Boolean;
begin
  {$ifopt C+}
  lCanvasClassName := aChart.Canvas.ClassName;
  {$endif}
  lCanvasClass := TCanvas3DClass(aChart.Canvas.ClassType);
  if lCanvasClass <> TTeeCanvas3D then
    aChart.Canvas := TTeeCanvas3D.Create;

  lHoldGradientVisible := aChart.Gradient.Visible;
  lHoldColor := aChart.Color;
  lHoldBevel := aChart.BevelOuter;
  lHoldShadow := aChart.Shadow.Visible;
  lHoldHover := aChart.Hover.Visible;

  try
    SetLength(lBoolArray, aChart.SeriesCount);
    for I := 0 to aChart.SeriesCount - 1 do    // Iterate
      if aChart.Series[I] is TPieSeries then
      begin
        lBoolArray[I] := TPieSeries(aChart.Series[I]).Circled;
        TPieSeries(aChart.Series[I]).Circled := True;
      end
      else
        lBoolArray[I] := True;


    aChart.Gradient.Visible  := false;
    if not aTransparent then
      aChart.Color := clWhite
    else
      aChart.Color := clNone; //clWhite;
    aChart.BevelOuter := bvNone;
    aChart.BevelInner := bvNone;
    aChart.Shadow.Visible := False;
    aChart.Hover.Visible := false;

    if aGraphic is TMetafile then
    begin
      lMetafile := aChart.TeeCreateMetafile(false, aRect);
      try
        aGraphic.Assign(lMetafile);
      finally
        lMetafile.Free;
      end;
    end
    else
    begin
      lBmp := aChart.TeeCreateBitmap(clWhite, aRect);
      try
        aGraphic.Assign(lBmp);
      finally
        lBmp.Free;
      end;
    end;
  finally
    aChart.Hover.Visible := lHoldHover;
    aChart.Shadow.Visible := lHoldShadow;
    aChart.Gradient.Visible := lHoldGradientVisible;
    aChart.Color := lHoldColor;
    aChart.BevelOuter := lHoldBevel;
    for I := 0 to aChart.SeriesCount - 1 do    // Iterate
      if aChart.Series[I] is TPieSeries then
        TPieSeries(aChart.Series[I]).Circled := lBoolArray[I];
    if (lCanvasClass <> aChart.Canvas.ClassType) then
      aChart.Canvas := lCanvasClass.Create;
  end;

  {$ifopt C+}
  assert(SameText(lCanvasClassName, aChart.Canvas.ClassName),
    Format('Chart canvas should be %s but is %s', [lCanvasClassName, aChart.Canvas.ClassName]));
  {$endif}
end;


procedure TForm1.btnToWMFClick(Sender: TObject);
begin
  RenderImage;
end;

procedure TForm1.RenderImage;
var
  lR: TRect;
  lPNG: TPNGImage;
begin
  lR := Rect(0, 0, Image1.Width, Image1.Height);

  lPNG :=  TPNGImage.Create;
  AssignChartToGraphic(Chart1, lR, lPNG);
  Image1.Picture.Graphic := lPNG;
  lPNG.Free;
end;

procedure TForm1.TabSheet2Show(Sender: TObject);
begin
  RenderImage;
end;

procedure TForm1.UpdateSeriesIcon;
var
  lNumValues: Integer;
  lIconValue: Integer;
  lFileName: string;
  lValue: double;
  I: integer;
begin
  if assigned(FsrsPoint) then
    FreeAndNil(FsrsPoint);

  lNumValues := spnNumValues.Value;
  lIconValue := Trunc(cltMarketHit.Value) + 1;

  if InRange(lIconValue, 1, lNumValues) then begin
    dec(lIconValue);

    FsrsPoint := Chart1.AddSeries(TImagePointSeries) as TImagePointSeries;

    FsrsPoint.Transparency := 50;
    with FsrsPoint do begin
      Title := '20%'#13'Market Hit';
      Title := 'Market Hit';
      Pointer.Size:= 64;
      lFileName := ExtractFilePath(ParamStr(0)) + 'lighting_bolt.png';
      Pointer.Picture.LoadFromFile(lFileName);
      Pointer.InflateMargins:=False;

      GetVertAxis.MaximumOffset := 32;

      lValue :=0;
      for i:=0 to Chart1.SeriesCount-1 do
        if Chart1[i] is TBarSeries then
           lValue:=lValue+Chart1[i].YValue[lIconValue];

      Chart1.Draw;

      with Chart1.Axes.Left do
        lValue:= CalcPosPoint(CalcPosValue(lValue) - Pointer.Size);

      AddXY(lIconValue, lValue, Title);
      Marks.Transparent := True;
    end;
  end;
end;

end.
